"""This module provides the main client and helper classes for interacting with the Mailgun API.

The `mailgun.client` module includes the core `Client` class for managing
API requests, configuration, and error handling, as well as utility functions
and classes for building request headers, URLs, and parsing responses.
Classes:
    - Config: Manages configuration settings for the Mailgun API.
    - Endpoint: Represents specific API endpoints and provides methods for
      common HTTP operations like GET, POST, PUT, and DELETE.
    - Client: The main API client for authenticating and making requests.
"""

from __future__ import annotations

import json
from typing import TYPE_CHECKING
from typing import Any
from urllib.parse import urljoin

import requests

from mailgun.handlers.default_handler import handle_default
from mailgun.handlers.domains_handler import handle_domainlist
from mailgun.handlers.domains_handler import handle_domains
from mailgun.handlers.domains_handler import handle_sending_queues
from mailgun.handlers.email_validation_handler import handle_address_validate
from mailgun.handlers.error_handler import ApiError
from mailgun.handlers.inbox_placement_handler import handle_inbox
from mailgun.handlers.ip_pools_handler import handle_ippools
from mailgun.handlers.ips_handler import handle_ips
from mailgun.handlers.mailinglists_handler import handle_lists
from mailgun.handlers.messages_handler import handle_resend_message
from mailgun.handlers.metrics_handler import handle_metrics
from mailgun.handlers.routes_handler import handle_routes
from mailgun.handlers.suppressions_handler import handle_bounces
from mailgun.handlers.suppressions_handler import handle_complaints
from mailgun.handlers.suppressions_handler import handle_unsubscribes
from mailgun.handlers.suppressions_handler import handle_whitelists
from mailgun.handlers.tags_handler import handle_tags
from mailgun.handlers.templates_handler import handle_templates


if TYPE_CHECKING:
    from collections.abc import Callable
    from collections.abc import Mapping

    from requests.models import Response


HANDLERS: dict[str, Callable] = {  # type: ignore[type-arg]
    "resendmessage": handle_resend_message,
    "domains": handle_domains,
    "domainlist": handle_domainlist,
    "dkim_authority": handle_domains,
    "dkim_selector": handle_domains,
    "web_prefix": handle_domains,
    "sending_queues": handle_sending_queues,
    "ips": handle_ips,
    "ip_pools": handle_ippools,
    "tags": handle_tags,
    "bounces": handle_bounces,
    "unsubscribes": handle_unsubscribes,
    "whitelists": handle_whitelists,
    "complaints": handle_complaints,
    "routes": handle_routes,
    "lists": handle_lists,
    "templates": handle_templates,
    "addressvalidate": handle_address_validate,
    "inbox": handle_inbox,
    "messages": handle_default,
    "messages.mime": handle_default,
    "events": handle_default,
    "analytics": handle_metrics,
}


class Config:
    """Config class. Configure client with basic (urls, version, headers)."""

    DEFAULT_API_URL: str = "https://api.mailgun.net/"
    API_REF: str = "https://documentation.mailgun.com/en/latest/api_reference.html"
    user_agent: str = "mailgun-api-python/"

    def __init__(self, api_url: str | None = None) -> None:
        """Initialize a new Config instance with specified or default API settings.

        This initializer sets the API version and base URL. If no version or URL
        is provided, it defaults to the predefined class values.

        :param version: API version (default: v3)
        :type version: str | None
        :param api_url: API base url
        :type api_url: str | None
        """
        self.ex_handler: bool = True
        self.api_url = api_url or self.DEFAULT_API_URL

    def __getitem__(self, key: str) -> tuple[Any, dict[str, str]]:
        """Parse incoming split attr name, check it and prepare endpoint url.

        Most urls generated here can't be generated dynamically as we are doing this
        in build_url() method under Endpoint class.
        :param key: incoming attr name
        :type key: str
        :return: url, headers
        """
        key = key.lower()
        headers = {"User-agent": self.user_agent}
        v1_base = urljoin(self.api_url, "v1/")
        v3_base = urljoin(self.api_url, "v3/")
        v4_base = urljoin(self.api_url, "v4/")
        v5_base = urljoin(self.api_url, "v5/")

        special_cases = {
            "messages": {"base": v3_base, "keys": ["messages"]},
            "mimemessage": {"base": v3_base, "keys": ["messages.mime"]},
            "resendmessage": {"base": v3_base, "keys": ["resendmessage"]},
            "ippools": {"base": v3_base, "keys": ["ip_pools"]},
            "dkimkeys": {"base": v1_base, "keys": ["dkim", "keys"]},
            "domainlist": {"base": v4_base, "keys": ["domainlist"]},
            # /v1/analytics/metrics
            # /v1/analytics/usage/metrics
            # /v1/analytics/logs
            # /v1/analytics/tags
            # /v1/analytics/tags/limits
            "analytics": {
                "base": v1_base,
                "keys": ["analytics", "usage", "metrics", "logs", "tags", "limits"],
            },
        }

        if key in special_cases:
            return special_cases[key], headers

        if "analytics" in key:
            headers |= {"Content-Type": "application/json"}
            return {
                "base": v1_base,
                "keys": key.split("_"),
            }, headers

        # Handle DIPP endpoints
        if "subaccount" in key:
            if "ip_pools" in key:
                return {
                    "base": v5_base,
                    "keys": ["accounts", "subaccounts", "ip_pools"],
                }, headers
            if "ip_pool" in key:
                return {
                    "base": v5_base,
                    "keys": ["accounts", "subaccounts", "{subaccountId}", "ip_pool"],
                }, headers

        # Handle DKIM management endpoints
        if "dkim_management" in key:
            if "rotation" in key:
                return {
                    "base": v1_base,
                    "keys": ["dkim_management", "domains", "{name}", "rotation"],
                }, headers
            if "rotate" in key:
                return {
                    "base": v1_base,
                    "keys": ["dkim_management", "domains", "{name}", "rotate"],
                }, headers

        if "domains" in key:
            split = key.split("_") if "_" in key else [key]
            final_keys = split

            if any(x in key for x in ("activate", "deactivate")):
                action = "activate" if "activate" in key else "deactivate"
                final_keys = [
                    "domains",
                    "{authority_name}",
                    "keys",
                    "{selector}",
                    action,
                ]
                return {"base": v4_base, "keys": final_keys}, headers

            if "dkimauthority" in split:
                final_keys = ["dkim_authority"]
            elif "dkimselector" in split:
                final_keys = ["dkim_selector"]
            elif "webprefix" in split:
                final_keys = ["web_prefix"]
            elif "sendingqueues" in split:
                final_keys = ["sending_queues"]

            v3_domain_endpoints = {
                "credentials",
                "connection",
                "tracking",
                "dkimauthority",
                "dkimselector",
                "webprefix",
                "webhooks",
                "sendingqueues",
            }
            base = v3_base if any(x in key for x in v3_domain_endpoints) else v4_base
            return {"base": f"{base}domains/", "keys": final_keys}, headers

        if "addressvalidate" in key:
            return {
                "base": f"{v4_base}address/validate",
                "keys": key.split("_"),
            }, headers

        return {"base": v3_base, "keys": key.split("_")}, headers


class Endpoint:
    """Generate request and return response."""

    def __init__(
        self,
        url: dict[str, Any],
        headers: dict[str, str],
        auth: tuple[str, str] | None,
    ) -> None:
        """Initialize a new Endpoint instance.

        :param url: URL dict with pairs {"base": "keys"}
        :type url: dict[str, Any]
        :param headers: Headers dict
        :type headers: dict[str, str]
        :param auth: requests auth tuple
        :type auth: tuple[str, str] | None
        """
        self._url = url
        self.headers = headers
        self._auth = auth

    def api_call(
        self,
        auth: tuple[str, str] | None,
        method: str,
        url: dict[str, Any],
        headers: dict[str, str],
        data: Any | None = None,
        filters: Mapping[str, str | Any] | None = None,
        timeout: int = 60,
        files: dict[str, bytes] | None = None,
        domain: str | None = None,
        **kwargs: Any,
    ) -> Response | Any:
        """Build URL and make a request.

        :param auth: auth data
        :type auth: tuple[str, str] | None
        :param method: request method
        :type method: str
        :param url: incoming url (base+keys)
        :type url: dict[str, Any]
        :param headers: incoming headers
        :type headers: dict[str, str]
        :param data: incoming post/put data
        :type data: Any | None
        :param filters: incoming params
        :type filters: dict | None
        :param timeout: requested timeout (60-default)
        :type timeout: int
        :param files: incoming files
        :type files: dict[str, Any] | None
        :param domain: incoming domain
        :type domain: str | None
        :param kwargs: kwargs
        :type kwargs: Any
        :return: server response from API
        :rtype: requests.models.Response
        :raises: TimeoutError, ApiError
        """
        url = self.build_url(url, domain=domain, method=method, **kwargs)
        req_method = getattr(requests, method)

        try:
            return req_method(
                url,
                data=data,
                params=filters,
                headers=headers,
                auth=auth,
                timeout=timeout,
                files=files,
                verify=True,
                stream=False,
            )

        except requests.exceptions.Timeout:
            raise TimeoutError
        except requests.RequestException as e:
            raise ApiError(e)
        except Exception as e:
            raise e

    @staticmethod
    def build_url(
        url: dict[str, Any],
        domain: str | None = None,
        method: str | None = None,
        **kwargs: Any,
    ) -> Any:
        """Build final request url using predefined handlers.

        Note: Some urls are being built in Config class, as they can't be generated dynamically.
        :param url: incoming url (base+keys)
        :type url: dict[str, Any]
        :param domain: incoming domain
        :type domain: str
        :param method: requested method
        :type method: str
        :param kwargs: kwargs
        :type kwargs: Any
        :return: built URL
        """
        return HANDLERS[url["keys"][0]](url, domain, method, **kwargs)

    def get(
        self,
        filters: Mapping[str, str | Any] | None = None,
        domain: str | None = None,
        **kwargs: Any,
    ) -> Response:
        """GET method for API calls.

        :param filters: incoming params
        :type filters: Mapping[str, str | Any] | None
        :param domain: incoming domain
        :type domain: str | None
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call GET request
        :rtype: requests.models.Response
        """
        return self.api_call(
            self._auth,
            "get",
            self._url,
            domain=domain,
            headers=self.headers,
            filters=filters,
            **kwargs,
        )

    def create(
        self,
        data: Any | None = None,
        filters: Mapping[str, str | Any] | None = None,
        domain: str | None = None,
        headers: str | None = None,
        files: dict[str, bytes] | None = None,
        **kwargs: Any,
    ) -> Response:
        """POST method for API calls.

        :param data: incoming post data
        :type data: Any | None
        :param filters: incoming params
        :type filters: dict
        :param domain: incoming domain
        :type domain: str
        :param headers: incoming headers
        :type headers: dict[str, str]
        :param files: incoming files
        :type files: dict[str, Any] | None
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call POST request
        :rtype: requests.models.Response
        """
        if "Content-Type" in self.headers:
            if self.headers["Content-Type"] == "application/json":
                data = json.dumps(data)
        elif headers:
            if headers == "application/json":
                data = json.dumps(data)
                self.headers["Content-Type"] = "application/json"
            elif headers == "multipart/form-data":
                self.headers["Content-Type"] = "multipart/form-data"

        return self.api_call(
            self._auth,
            "post",
            self._url,
            files=files,
            domain=domain,
            headers=self.headers,
            data=data,
            filters=filters,
            **kwargs,
        )

    def put(
        self,
        data: Any | None = None,
        filters: Mapping[str, str | Any] | None = None,
        **kwargs: Any,
    ) -> Response:
        """PUT method for API calls.

        :param data: incoming data
        :type data: Any | None
        :param filters: incoming params
        :type filters: dict
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call PUT request
        :rtype: requests.models.Response
        """
        return self.api_call(
            self._auth,
            "put",
            self._url,
            headers=self.headers,
            data=data,
            filters=filters,
            **kwargs,
        )

    def patch(
        self,
        data: Any | None = None,
        filters: Mapping[str, str | Any] | None = None,
        **kwargs: Any,
    ) -> Response:
        """PATCH method for API calls.

        :param data: incoming data
        :type data: Any | None
        :param filters: incoming params
        :type filters: dict
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call PATCH request
        :rtype: requests.models.Response
        """
        return self.api_call(
            self._auth,
            "patch",
            self._url,
            headers=self.headers,
            data=data,
            filters=filters,
            **kwargs,
        )

    def update(
        self,
        data: Any | None,
        filters: Mapping[str, str | Any] | None = None,
        **kwargs: Any,
    ) -> Response:
        """PUT method for API calls.

        :param data: incoming data
        :type data: dict[str, Any] | None
        :param filters: incoming params
        :type filters: dict
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call PUT request
        :rtype: requests.models.Response
        """
        if self.headers["Content-type"] == "application/json":
            data = json.dumps(data)
        return self.api_call(
            self._auth,
            "put",
            self._url,
            headers=self.headers,
            data=data,
            filters=filters,
            **kwargs,
        )

    def delete(self, domain: str | None = None, **kwargs: Any) -> Response:
        """DELETE method for API calls.

        :param domain: incoming domain
        :type domain: str
        :param kwargs: kwargs
        :type kwargs: Any
        :return: api_call DELETE request
        :rtype: requests.models.Response
        """
        return self.api_call(
            self._auth,
            "delete",
            self._url,
            headers=self.headers,
            domain=domain,
            **kwargs,
        )


class Client:
    """Client class."""

    def __init__(self, auth: tuple[str, str] | None = None, **kwargs: Any) -> None:
        """Initialize a new Client instance for API interaction.

        This method sets up API authentication and configuration. The `auth` parameter
        provides a tuple with the API key and secret. Additional keyword arguments can
        specify configuration options like API version and URL.

        :param auth: auth set ("username", "APIKEY")
        :type auth: set
        :param kwargs: kwargs
        """
        self.auth = auth
        api_url = kwargs.get("api_url")
        self.config = Config(api_url=api_url)

    def __getattr__(self, name: str) -> Any:
        """Get named attribute of an object, split it and execute.

        :param name: attribute name (Example: client.domains_ips. names: ["domains", "ips"])
        :type name: str
        :return: type object (executes existing handler)
        """
        split = name.split("_")
        # identify the resource
        fname = split[0]
        url, headers = self.config[name]
        return type(fname, (Endpoint,), {})(url=url, headers=headers, auth=self.auth)



import os
from pathlib import Path

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]
html: str = """<body style="margin: 0; padding: 0;">
 <table border="1" cellpadding="0" cellspacing="0" width="100%">
  <tr>
   <td>
    Hello!
   </td>
  </tr>
 </table>
</body>"""
client: Client = Client(auth=("api", key))


def post_message() -> None:
    # Messages
    # POST /<domain>/messages
    data = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "cc": os.environ["MESSAGES_CC"],
        "subject": "Hello Vasyl Bodaj",
        "html": html,
        "o:tag": "Python test",
    }
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = [
        (
            "attachment",
            ("test1.txt", Path("mailgun/doc_tests/files/test1.txt").read_bytes()),
        ),
        (
            "attachment",
            ("test2.txt", Path("mailgun/doc_tests/files/test2.txt").read_bytes()),
        ),
    ]

    req = client.messages.create(data=data, files=files, domain=domain)
    print(req.json())


def post_mime() -> None:
    # Mime messages
    # POST /<domain>/messages.mime
    mime_data = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "cc": os.environ["MESSAGES_CC"],
        "subject": "Hello HELLO",
    }
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {"message": Path("mailgun/doc_tests/files/test_mime.mime").read_bytes()}

    req = client.mimemessage.create(data=mime_data, files=files, domain=domain)
    print(req.json())


def post_no_tracking() -> None:
    # Message no tracking
    data = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "cc": os.environ["MESSAGES_CC"],
        "subject": "Hello Vasyl Bodaj",
        "html": html,
        "o:tracking": False,
    }

    req = client.messages.create(data=data, domain=domain)
    print(req.json())


def post_scheduled() -> None:
    # Scheduled message
    data = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "cc": os.environ["MESSAGES_CC"],
        "subject": "Hello Vasyl Bodaj",
        "html": html,
        "o:deliverytime": "Thu Jan 28 2021 14:00:03 EST",
    }

    req = client.messages.create(data=data, domain=domain)
    print(req.json())


def post_message_tags() -> None:
    # Message Tags
    data = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "cc": os.environ["MESSAGES_CC"],
        "subject": "Hello Vasyl Bodaj",
        "html": html,
        "o:tag": ["September newsletter", "newsletters"],
    }

    req = client.messages.create(data=data, domain=domain)
    print(req.json())


def resend_message() -> None:
    data = {"to": ["spidlisn@gmail.com", "mailgun@2048.zeefarmer.com"]}

    params = {
        "from": os.environ["MESSAGES_FROM"],
        "to": os.environ["MESSAGES_TO"],
        "limit": 1,
    }
    req_ev = client.events.get(domain=domain, filters=params)
    print(req_ev.json())

    req = client.resendmessage.create(
        data=data,
        domain=domain,
        storage_url=req_ev.json()["items"][0]["storage"]["url"],
    )
    print(req.json())


if __name__ == "__main__":
    post_message()



"""EMAIL VALIDATION HANDLER.

Doc: https://documentation.mailgun.com/en/latest/api-email-validation.html#email-validation
"""

from __future__ import annotations

from os import path
from typing import Any


def handle_address_validate(
    url: dict[str, Any],
    _domain: str | None,
    _method: str | None,
    **kwargs: Any,
) -> Any:
    """Handle email validation.

    :param url: Incoming URL dictionary
    :type url: dict
    :param _domain: Incoming domain (it's not being used for this handler)
    :type _domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for email validation endpoint
    """
    final_keys = path.join("/", *url["keys"][1:]) if url["keys"][1:] else ""
    if "list_name" in kwargs:
        url = url["base"] + final_keys + "/" + kwargs["list_name"]
    else:
        url = url["base"] + final_keys

    return url



import os

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))


def get_ippools() -> None:
    """
    GET /v1/ip_pools
    :return:
    """
    req = client.ippools.get(domain=domain)
    print(req.json())


def create_ippool() -> None:
    """
    POST /v1/ip_pools
    :return:
    """
    post_data = {"name": "test_pool1", "description": "Test", "ips": ["166.78.68.186"]}
    req_post = client.ippools.create(domain=domain, data=post_data)
    print(req_post.json())


def update_ippool() -> None:
    """
    PATCH /v1/ip_pools/{pool_id}
    :return:
    """
    data = {
        "name": "test_pool3",
        "description": "Test3",
    }
    req = client.ippools.patch(domain=domain, data=data, pool_id="60140bc1fee3e84dec5abeeb")
    print(req.json())


def delete_ippool() -> None:
    """
    DELETE /v1/ip_pools/{pool_id}
    :return:
    """
    req = client.ippools.delete(domain=domain, pool_id="60140bc1fee3e84dec5abeeb")
    print(req.json())


def link_ippool() -> None:
    """
    POST /v3/domains/{domain_name}/ips
    :return:
    """
    data = {"pool_id": "60140d220859fda7bab8bb6c"}
    req = client.domains_ips.create(domain=domain, data=data)
    print(req.json())


def unlink_ippool() -> None:
    """
    DELETE /v3/domains/{domain_name}/ips/ip_pool
    :return:
    """
    data = {"pool_id": "5ff37204e5eb74149462c375"}

    req = client.domains_ips.delete(domain=domain, filters=data, unlink_pool=True)
    print(req.json())


if __name__ == "__main__":
    get_ippools()



"""DEFAULT HANDLER.

Events doc: https://documentation.mailgun.com/en/latest/api-events.html
Messages doc: https://documentation.mailgun.com/en/latest/api-sending.html
Stats doc: https://documentation.mailgun.com/en/latest/api-stats.html
"""

from __future__ import annotations

from os import path
from typing import Any

from .error_handler import ApiError


def handle_default(
    url: dict[str, Any],
    domain: str | None,
    _method: str | None,
    **_: Any,
) -> Any:
    """Provide default handler for endpoints with single url pattern (events, messages, stats).

    :param url: Incoming URL dictionary
    :type url: dict
    :param domain: Incoming domain
    :type domain: str
    :param _method: Incoming request method (it's not being used for this handler)
    :type _method: str
    :param kwargs: kwargs
    :return: final url for default endpoint
    :raises: ApiError
    """
    if not domain:
        raise ApiError("Domain is missing!")

    final_keys = path.join("/", *url["keys"]) if url["keys"] else ""
    return url["base"] + domain + final_keys



import os
from pathlib import Path

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))


def get_single_validate() -> None:
    """
    GET /v4/address/validate
    :return:
    """
    params = {"address": "test@gmail.com", "provider_lookup": "false"}
    req = client.addressvalidate.get(domain=domain, filters=params)
    print(req.json())


def post_single_validate() -> None:
    """
    POST /v4/address/validate
    :return:
    """
    data = {"address": "test2@gmail.com"}
    params = {"provider_lookup": "false"}
    req = client.addressvalidate.create(domain=domain, data=data, filters=params)
    print(req.json())


def get_bulk_validate() -> None:
    """
    GET /v4/address/validate/bulk
    :return:
    """
    params = {"limit": 2}
    req = client.addressvalidate_bulk.get(domain=domain, filters=params)
    print(req.json())


def post_bulk_list_validate() -> None:
    """
    POST /v4/address/validate/bulk/<list_id>
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {"file": Path("mailgun/doc_tests/files/email_validation.csv").read_bytes()}
    req = client.addressvalidate_bulk.create(domain=domain, files=files, list_name="python2_list")
    print(req.json())


def get_bulk_list_validate() -> None:
    """
    GET /v4/address/validate/bulk/<list_id>
    :return:
    """
    req = client.addressvalidate_bulk.get(domain=domain, list_name="python2_list")
    print(req.json())


def delete_bulk_list_validate() -> None:
    """
    DELETE /v4/address/validate/bulk/<list_id>
    :return:
    """
    req = client.addressvalidate_bulk.delete(domain=domain, list_name="python2_list")
    print(req.json())


def get_preview() -> None:
    """
    GET /v4/address/validate/preview
    :return:
    """
    req = client.addressvalidate_preview.get(domain=domain)
    print(req.json())


def post_preview() -> None:
    """
    POST /v4/address/validate/preview/<list_id>
    :return:
    """
    # It is strongly recommended that you open files in binary mode.
    # Because the Content-Length header may be provided for you,
    # and if it does this value will be set to the number of bytes in the file.
    # Errors may occur if you open the file in text mode.
    files = {"file": Path("mailgun/doc_tests/files/email_previews.csv").read_bytes()}
    req = client.addressvalidate_preview.create(domain=domain, files=files, list_name="python_list")
    print(req.json())


def delete_preview() -> None:
    """
    DELETE /v4/address/validate/preview/<list_id>
    :return:
    """
    req = client.addressvalidate_preview.delete(domain=domain, list_name="python_list")
    print(req.text)


if __name__ == "__main__":
    delete_preview()



import os

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]
client: Client = Client(auth=("api", key))


def post_analytics_metrics() -> None:
    """
    # Metrics
    # POST /v1/analytics/metrics
    :return:
    """

    data = {
        "start": "Sun, 08 Jun 2025 00:00:00 +0000",
        "end": "Tue, 08 Jul 2025 00:00:00 +0000",
        "resolution": "day",
        "duration": "1m",
        "dimensions": ["time"],
        "metrics": ["accepted_count", "delivered_count", "clicked_rate", "opened_rate"],
        "filter": {
            "AND": [
                {
                    "attribute": "domain",
                    "comparator": "=",
                    "values": [{"label": domain, "value": domain}],
                }
            ]
        },
        "include_subaccounts": True,
        "include_aggregates": True,
    }

    req = client.analytics_metrics.create(data=data)
    print(req.json())


def post_analytics_usage_metrics() -> None:
    """
    # Usage Metrics
    # POST /v1/analytics/usage/metrics
    :return:
    """
    data = {
        "start": "Sun, 08 Jun 2025 00:00:00 +0000",
        "end": "Tue, 08 Jul 2025 00:00:00 +0000",
        "resolution": "day",
        "duration": "1m",
        "dimensions": ["time"],
        "metrics": [
            "accessibility_count",
            "accessibility_failed_count",
            "domain_blocklist_monitoring_count",
            "email_preview_count",
            "email_preview_failed_count",
            "email_validation_bulk_count",
            "email_validation_count",
            "email_validation_list_count",
            "email_validation_mailgun_count",
            "email_validation_mailjet_count",
            "email_validation_public_count",
            "email_validation_single_count",
            "email_validation_valid_count",
            "image_validation_count",
            "image_validation_failed_count",
            "ip_blocklist_monitoring_count",
            "link_validation_count",
            "link_validation_failed_count",
            "processed_count",
            "seed_test_count",
        ],
        "include_subaccounts": True,
        "include_aggregates": True,
    }

    req = client.analytics_usage_metrics.create(data=data)
    print(req.json())


if __name__ == "__main__":
    post_analytics_metrics()
    post_analytics_usage_metrics()



from __future__ import annotations

import os

from mailgun.client import Client


key: str = os.environ["APIKEY"]
domain: str = os.environ["DOMAIN"]

client: Client = Client(auth=("api", key))


def get_domains() -> None:
    """
    GET /domains
    :return:
    """
    data = client.domainlist.get()
    print(data.json())


def add_domain() -> None:
    """
    POST /domains
    :return:
    """
    # Post domain
    data = {
        "name": "python.test.domain5",
    }
    # Problem with smtp_password!!!!

    request = client.domains.create(data=data)
    print(request.json())
    print(request.status_code)


# Get domain


def get_simple_domain() -> None:
    """
    GET /domains/<domain>
    :return:
    """
    domain_name = "python.test.domain5"
    request = client.domains.get(domain_name=domain_name)
    print(request.json())


def update_simple_domain() -> None:
    """
    PUT /domains/<domain>
    :return:
    """
    domain_name = "python.test.domain5"
    data = {"name": domain_name, "spam_action": "disabled"}
    request = client.domains.put(data=data, domain=domain_name)
    print(request.json())


def verify_domain() -> None:
    """
    PUT /domains/<domain>/verify
    :return:
    """
    domain_name = "python.test.domain5"
    request = client.domains.put(domain=domain_name, verify=True)
    print(request.json())


def delete_domain() -> None:
    """
    DELETE /domains/<domain>
    :return:
    """
    # Delete domain
    request = client.domains.delete(domain="python.test.domain5")
    print(request.text)
    print(request.status_code)


def get_credentials() -> None:
    """
    GET /domains/<domain>/credentials
    :return:
    """
    request = client.domains_credentials.get(domain=domain)
    print(request.json())


def post_credentials() -> None:
    """
    POST /domains/<domain>/credentials
    :return:
    """
    data = {
        "login": f"alice_bob@{domain}",
        "password": "test_new_creds123",  # pragma: allowlist secret
    }
    request = client.domains_credentials.create(domain=domain, data=data)
    print(request.json())


def put_credentials() -> None:
    """
    PUT /domains/<domain>/credentials/<login>
    :return:
    """
    data = {"password": "test_new_creds12356"}  # pragma: allowlist secret
    request = client.domains_credentials.put(domain=domain, data=data, login=f"alice_bob@{domain}")
    print(request.json())


def delete_all_domain_credentials() -> None:
    """
    DELETE /domains/<domain>/credentials
    :return:
    """
    request = client.domains_credentials.delete(domain=domain)
    print(request.json())


def delete_credentials() -> None:
    """
    DELETE /domains/<domain>/credentials/<login>
    :return:
    """
    request = client.domains_credentials.delete(domain=domain, login=f"alice_bob@{domain}")
    print(request.json())


def get_connections() -> None:
    """
    GET /domains/<domain>/connection
    :return:
    """
    request = client.domains_connection.get(domain=domain)
    print(request.json())


def put_connections() -> None:
    """
    PUT /domains/<domain>/connection
    :return:
    """
    data = {"require_tls": "true", "skip_verification": "false"}
    request = client.domains_connection.put(domain=domain, data=data)
    print(request.json())


def get_tracking() -> None:
    """
    GET /domains/<domain>/tracking
    :return:
    """
    request = client.domains_tracking.get(domain=domain)
    print(request.json())


def put_open_tracking() -> None:
    """
    PUT /domains/<domain>/tracking/open
    :return:
    """
    data = {"active": "yes", "skip_verification": "false"}
    request = client.domains_tracking_open.put(domain=domain, data=data)
    print(request.json())


def put_click_tracking() -> None:
    """
    PUT /domains/<domain>/tracking/click
    :return:
    """
    data = {
        "active": "yes",
    }
    request = client.domains_tracking_click.put(domain=domain, data=data)
    print(request.json())


def put_unsub_tracking() -> None:
    """
    PUT /domains/<domain>/tracking/unsubscribe
    :return:
    """
    # fmt: off
    data = {
        "active": "yes",
        "html_footer": "\n<br>\n<p><a href=\"%unsubscribe_url%\">UnSuBsCrIbE</a></p>\n",
        "text_footer": "\n\nTo unsubscribe here click: <%unsubscribe_url%>\n\n"
    }
    # fmt: on
    request = client.domains_tracking_unsubscribe.put(domain=domain, data=data)
    print(request.json())


def put_dkim_authority() -> None:
    """
    PUT /domains/<domain>/dkim_authority
    :return:
    """
    data = {"self": "false"}
    request = client.domains_dkimauthority.put(domain=domain, data=data)
    print(request.json())


def put_dkim_selector() -> None:
    """
    PUT /domains/<domain>/dkim_selector
    :return:
    """
    data = {"dkim_selector": "s"}
    request = client.domains_dkimselector.put(domain="python.test.domain5", data=data)
    print(request.json())


def put_web_prefix() -> None:
    """
    PUT /domains/<domain>/web_prefix
    :return:
    """
    data = {"web_prefix": "python"}
    request = client.domains_webprefix.put(domain="python.test.domain5", data=data)
    print(request.json())


def get_sending_queues() -> None:
    """
    GET /domains/<domain>/sending_queues
    :return:
    """
    request = client.domains_sendingqueues.get(domain="python.test.domain5")
    print(request)
    print(request.json())


if __name__ == "__main__":
    get_domains()
